﻿##### Scriptinfo ##################################################################################
# Scriptreihe:      Show-FileWritePerformance
# Datum:            2018-10-05
# Version:          V1.01
# Programmierer:    Stephan Walther
###################################################################################################

# Variablen
$Pfad   = "E:\"
$Anzahl = 100000   
$Text   = "Beispieltext" 
    
    Set-Location -Path $Pfad
    cls 

# 1. Versuchsreihe 
    # >> Operator 
        $Duration = (Measure-Command -Expression {
            1..$Anzahl | ForEach-Object { 
                 $Text >> "$Pfad\test1.txt" 
            } 
        }).TotalSeconds
        ">> Operator:  $Duration"

    # Out-File #1
        $Duration = (Measure-Command -Expression {
            1..$Anzahl | ForEach-Object {$Text} | Out-File -FilePath "$Pfad\test2a.txt" -Encoding utf8
        }).TotalSeconds
        "Out-File #1:  $Duration"

    # Out-File #2
        $Duration = (Measure-Command -Expression {
            (1..$Anzahl | ForEach-Object {$Text}) -join "`r`n" | Out-File -FilePath "$Pfad\test2b.txt" -Encoding utf8
        }).TotalSeconds
        "Out-File #2:  $Duration"
        
    # Export-CSV
        $Duration = (Measure-Command -Expression {
            (1..$Anzahl | ForEach-Object {$Text}) -join "`r`n" | ConvertFrom-Csv | Export-Csv -Path "$Pfad\test3.txt" -NoTypeInformation -Encoding UTF8 -UseCulture
         
        }).TotalSeconds
        "Export-Csv:   $Duration"

    # Streamwriter
        $Duration = (Measure-Command -Expression {
            $Textstream = [System.IO.StreamWriter] "$Pfad\test4.txt"
            1..$Anzahl | ForEach-Object {
                  $Textstream.WriteLine($Text)
            }
            $Textstream.close()
        }).TotalSeconds
        "StreamWriter: $Duration"

    # CleanUp
        Get-ChildItem -Path "$Pfad\test??.txt" | Remove-Item -Confirm

# 2. Versuchsreihe
    # generiere Variable
        $Duration = (Measure-Command -Expression {
            $Daten = 1..$Anzahl | ForEach-Object {$Text}
        }).TotalSeconds
        "erstelle Var: $Duration" 
        ""
    # >> Operator 
        $Duration = (Measure-Command -Expression {
            $Daten >> "$Pfad\test1.txt"             
        }).TotalSeconds
        ">> Operator:  $Duration"

    # Out-File #1
        $Duration = (Measure-Command -Expression {
            $Daten | Out-File -FilePath "$Pfad\test2a.txt" -Encoding utf8
        }).TotalSeconds
        "Out-File #1:  $Duration"

    # Out-File #2
        $Duration = (Measure-Command -Expression {
            $Daten -join "`r`n" | Out-File -FilePath "$Pfad\test2b.txt" -Encoding utf8
        }).TotalSeconds
        "Out-File #2:  $Duration"
        
    # Export-CSV
        $Duration = (Measure-Command -Expression {
            $Daten -join "`r`n" | ConvertFrom-Csv | Export-Csv -Path "$Pfad\test3.txt" -NoTypeInformation -Encoding UTF8 -UseCulture
         
        }).TotalSeconds
        "Export-Csv:   $Duration"

    # Streamwriter
        $Duration = (Measure-Command -Expression {
            $Textstream = [System.IO.StreamWriter] "$Pfad\test4.txt"
            $Textstream.Write($Daten)            
            $Textstream.close()
        }).TotalSeconds
        "StreamWriter: $Duration"

    # CleanUp
        Get-ChildItem -Path "$Pfad\test??.txt" | Remove-Item -Confirm

# Maximierung der Performance
    # Optimierung der Variablen-Generierung
        $Duration = (Measure-Command -Expression {
            $TextMitZeilenumbruch = $Text + "`r`n"
            $Daten = $TextMitZeilenumbruch * $Anzahl
        }).TotalSeconds
        "erstelle Var: $Duration" 

    # Finale Lösung
        $Duration = (Measure-Command -Expression {
            $TextMitZeilenumbruch = $Text + "`r`n"
            $Daten = $TextMitZeilenumbruch * $Anzahl

            $Textstream = [System.IO.StreamWriter] "$Pfad\test4.txt"
            $Textstream.Write($Daten)
            $Textstream.close()

        }).TotalSeconds
        "StreamWriter: $Duration" 
